@@ -42,7 +42,7 @@ void ngtcp2_log_init(ngtcp2_log *log, const ngtcp2_cid *scid,
                      ngtcp2_printf log_printf, ngtcp2_tstamp ts,
                      void *user_data) {
   if (scid) {
-    ngtcp2_encode_hex(log->scid, scid->data, scid->datalen);
+    ngtcp2_encode_hex_cstr(log->scid, scid->data, scid->datalen);
   } else {
     log->scid[0] = '\0';
   }
@@ -127,6 +127,10 @@ static const char *strerrorcode(uint64_t error_code) {
     return "CRYPTO_BUFFER_EXCEEDED";
   case NGTCP2_KEY_UPDATE_ERROR:
     return "KEY_UPDATE_ERROR";
+  case NGTCP2_AEAD_LIMIT_REACHED:
+    return "AEAD_LIMIT_REACHED";
+  case NGTCP2_NO_VIABLE_PATH:
+    return "NO_VIABLE_PATH";
   case NGTCP2_VERSION_NEGOTIATION_ERROR:
     return "VERSION_NEGOTIATION_ERROR";
   default:
@@ -267,7 +271,7 @@ static void log_fr_connection_close(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
     fr->type == NGTCP2_FRAME_CONNECTION_CLOSE ? strerrorcode(fr->error_code)
                                               : strapperrorcode(fr->error_code),
     fr->error_code, fr->frame_type, fr->reasonlen,
-    ngtcp2_encode_printable_ascii(reason, fr->reason, reasonlen));
+    ngtcp2_encode_printable_ascii_cstr(reason, fr->reason, reasonlen));
 }
 
 static void log_fr_max_data(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
@@ -335,8 +339,8 @@ static void log_fr_streams_blocked(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
 static void log_fr_new_connection_id(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
                                      const ngtcp2_new_connection_id *fr,
                                      const char *dir) {
-  uint8_t buf[sizeof(fr->stateless_reset_token) * 2 + 1];
-  uint8_t cid[sizeof(fr->cid.data) * 2 + 1];
+  char buf[sizeof(fr->stateless_reset_token) * 2 + 1];
+  char cid[sizeof(fr->cid.data) * 2 + 1];
 
   ngtcp2_log_infof_raw(
     log, NGTCP2_LOG_EVENT_FRM,
@@ -344,10 +348,10 @@ static void log_fr_new_connection_id(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
                    " cid=0x%s retire_prior_to=%" PRIu64
                    " stateless_reset_token=0x%s",
     NGTCP2_LOG_PKT_HD_FIELDS(dir), fr->type, fr->seq,
-    (const char *)ngtcp2_encode_hex(cid, fr->cid.data, fr->cid.datalen),
+    ngtcp2_encode_hex_cstr(cid, fr->cid.data, fr->cid.datalen),
     fr->retire_prior_to,
-    (const char *)ngtcp2_encode_hex(buf, fr->stateless_reset_token,
-                                    sizeof(fr->stateless_reset_token)));
+    ngtcp2_encode_hex_cstr(buf, fr->stateless_reset_token,
+                           sizeof(fr->stateless_reset_token)));
 }
 
 static void log_fr_stop_sending(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
@@ -364,25 +368,25 @@ static void log_fr_stop_sending(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
 static void log_fr_path_challenge(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
                                   const ngtcp2_path_challenge *fr,
                                   const char *dir) {
-  uint8_t buf[sizeof(fr->data) * 2 + 1];
+  char buf[sizeof(fr->data) * 2 + 1];
 
-  ngtcp2_log_infof_raw(
-    log, NGTCP2_LOG_EVENT_FRM,
-    NGTCP2_LOG_PKT " PATH_CHALLENGE(0x%02" PRIx64 ") data=0x%s",
-    NGTCP2_LOG_PKT_HD_FIELDS(dir), fr->type,
-    (const char *)ngtcp2_encode_hex(buf, fr->data, sizeof(fr->data)));
+  ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_FRM,
+                       NGTCP2_LOG_PKT " PATH_CHALLENGE(0x%02" PRIx64
+                                      ") data=0x%s",
+                       NGTCP2_LOG_PKT_HD_FIELDS(dir), fr->type,
+                       ngtcp2_encode_hex_cstr(buf, fr->data, sizeof(fr->data)));
 }
 
 static void log_fr_path_response(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
                                  const ngtcp2_path_response *fr,
                                  const char *dir) {
-  uint8_t buf[sizeof(fr->data) * 2 + 1];
+  char buf[sizeof(fr->data) * 2 + 1];
 
-  ngtcp2_log_infof_raw(
-    log, NGTCP2_LOG_EVENT_FRM,
-    NGTCP2_LOG_PKT " PATH_RESPONSE(0x%02" PRIx64 ") data=0x%s",
-    NGTCP2_LOG_PKT_HD_FIELDS(dir), fr->type,
-    (const char *)ngtcp2_encode_hex(buf, fr->data, sizeof(fr->data)));
+  ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_FRM,
+                       NGTCP2_LOG_PKT " PATH_RESPONSE(0x%02" PRIx64
+                                      ") data=0x%s",
+                       NGTCP2_LOG_PKT_HD_FIELDS(dir), fr->type,
+                       ngtcp2_encode_hex_cstr(buf, fr->data, sizeof(fr->data)));
 }
 
 static void log_fr_crypto(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
@@ -398,21 +402,21 @@ static void log_fr_new_token(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
                              const ngtcp2_new_token *fr, const char *dir) {
   /* Show at most first 64 bytes of token.  If token is longer than 64
      bytes, log first 64 bytes and then append "*" */
-  uint8_t buf[128 + 1 + 1];
-  uint8_t *p;
+  char buf[128 + 1 + 1];
+  char *p;
 
   if (fr->tokenlen > 64) {
-    p = ngtcp2_encode_hex(buf, fr->token, 64);
+    p = ngtcp2_encode_hex_cstr(buf, fr->token, 64);
     p[128] = '*';
     p[129] = '\0';
   } else {
-    p = ngtcp2_encode_hex(buf, fr->token, fr->tokenlen);
+    p = ngtcp2_encode_hex_cstr(buf, fr->token, fr->tokenlen);
   }
 
   ngtcp2_log_infof_raw(
     log, NGTCP2_LOG_EVENT_FRM,
     NGTCP2_LOG_PKT " NEW_TOKEN(0x%02" PRIx64 ") token=0x%s len=%zu",
-    NGTCP2_LOG_PKT_HD_FIELDS(dir), fr->type, (const char *)p, fr->tokenlen);
+    NGTCP2_LOG_PKT_HD_FIELDS(dir), fr->type, p, fr->tokenlen);
 }
 
 static void log_fr_retire_connection_id(ngtcp2_log *log,
@@ -443,7 +447,7 @@ static void log_fr_datagram(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
 
 static void log_fr(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
                    const ngtcp2_frame *fr, const char *dir) {
-  switch (fr->type) {
+  switch (fr->hd.type) {
   case NGTCP2_FRAME_STREAM:
     log_fr_stream(log, hd, &fr->stream, dir);
     break;
@@ -550,7 +554,7 @@ void ngtcp2_log_rx_vn(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
 }
 
 void ngtcp2_log_rx_sr(ngtcp2_log *log, const ngtcp2_pkt_stateless_reset *sr) {
-  uint8_t buf[sizeof(sr->stateless_reset_token) * 2 + 1];
+  char buf[sizeof(sr->stateless_reset_token) * 2 + 1];
   ngtcp2_pkt_hd shd;
   ngtcp2_pkt_hd *hd = &shd;
 
@@ -565,16 +569,16 @@ void ngtcp2_log_rx_sr(ngtcp2_log *log, const ngtcp2_pkt_stateless_reset *sr) {
   ngtcp2_log_infof_raw(
     log, NGTCP2_LOG_EVENT_PKT, NGTCP2_LOG_PKT " token=0x%s randlen=%zu",
     NGTCP2_LOG_PKT_HD_FIELDS("rx"),
-    (const char *)ngtcp2_encode_hex(buf, sr->stateless_reset_token,
-                                    sizeof(sr->stateless_reset_token)),
+    ngtcp2_encode_hex_cstr(buf, sr->stateless_reset_token,
+                           sizeof(sr->stateless_reset_token)),
     sr->randlen);
 }
 
 void ngtcp2_log_remote_tp(ngtcp2_log *log,
                           const ngtcp2_transport_params *params) {
-  uint8_t token[NGTCP2_STATELESS_RESET_TOKENLEN * 2 + 1];
-  uint8_t addr[16 * 2 + 7 + 1];
-  uint8_t cid[NGTCP2_MAX_CIDLEN * 2 + 1];
+  char token[NGTCP2_STATELESS_RESET_TOKENLEN * 2 + 1];
+  char addr[16 * 2 + 7 + 1];
+  char cid[NGTCP2_MAX_CIDLEN * 2 + 1];
   size_t i;
   const ngtcp2_sockaddr_in *sa_in;
   const ngtcp2_sockaddr_in6 *sa_in6;
@@ -588,18 +592,18 @@ void ngtcp2_log_remote_tp(ngtcp2_log *log,
   if (params->stateless_reset_token_present) {
     ngtcp2_log_infof_raw(
       log, NGTCP2_LOG_EVENT_CRY, NGTCP2_LOG_TP " stateless_reset_token=0x%s",
-      (const char *)ngtcp2_encode_hex(token, params->stateless_reset_token,
-                                      sizeof(params->stateless_reset_token)));
+      ngtcp2_encode_hex_cstr(token, params->stateless_reset_token,
+                             sizeof(params->stateless_reset_token)));
   }
 
   if (params->preferred_addr_present) {
     if (params->preferred_addr.ipv4_present) {
       sa_in = &params->preferred_addr.ipv4;
 
-      ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_CRY,
-                           NGTCP2_LOG_TP " preferred_address.ipv4_addr=%s",
-                           (const char *)ngtcp2_encode_ipv4(
-                             addr, (const uint8_t *)&sa_in->sin_addr));
+      ngtcp2_log_infof_raw(
+        log, NGTCP2_LOG_EVENT_CRY,
+        NGTCP2_LOG_TP " preferred_address.ipv4_addr=%s",
+        ngtcp2_encode_ipv4_cstr(addr, (const uint8_t *)&sa_in->sin_addr));
       ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_CRY,
                            NGTCP2_LOG_TP " preferred_address.ipv4_port=%u",
                            ngtcp2_ntohs(sa_in->sin_port));
@@ -608,10 +612,10 @@ void ngtcp2_log_remote_tp(ngtcp2_log *log,
     if (params->preferred_addr.ipv6_present) {
       sa_in6 = &params->preferred_addr.ipv6;
 
-      ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_CRY,
-                           NGTCP2_LOG_TP " preferred_address.ipv6_addr=%s",
-                           (const char *)ngtcp2_encode_ipv6(
-                             addr, (const uint8_t *)&sa_in6->sin6_addr));
+      ngtcp2_log_infof_raw(
+        log, NGTCP2_LOG_EVENT_CRY,
+        NGTCP2_LOG_TP " preferred_address.ipv6_addr=%s",
+        ngtcp2_encode_ipv6_cstr(addr, (const uint8_t *)&sa_in6->sin6_addr));
       ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_CRY,
                            NGTCP2_LOG_TP " preferred_address.ipv6_port=%u",
                            ngtcp2_ntohs(sa_in6->sin6_port));
@@ -619,38 +623,36 @@ void ngtcp2_log_remote_tp(ngtcp2_log *log,
 
     ngtcp2_log_infof_raw(
       log, NGTCP2_LOG_EVENT_CRY, NGTCP2_LOG_TP " preferred_address.cid=0x%s",
-      (const char *)ngtcp2_encode_hex(cid, params->preferred_addr.cid.data,
-                                      params->preferred_addr.cid.datalen));
+      ngtcp2_encode_hex_cstr(cid, params->preferred_addr.cid.data,
+                             params->preferred_addr.cid.datalen));
     ngtcp2_log_infof_raw(
       log, NGTCP2_LOG_EVENT_CRY,
       NGTCP2_LOG_TP " preferred_address.stateless_reset_token=0x%s",
-      (const char *)ngtcp2_encode_hex(
+      ngtcp2_encode_hex_cstr(
         token, params->preferred_addr.stateless_reset_token,
         sizeof(params->preferred_addr.stateless_reset_token)));
   }
 
   if (params->original_dcid_present) {
-    ngtcp2_log_infof_raw(
-      log, NGTCP2_LOG_EVENT_CRY,
-      NGTCP2_LOG_TP " original_destination_connection_id=0x%s",
-      (const char *)ngtcp2_encode_hex(cid, params->original_dcid.data,
-                                      params->original_dcid.datalen));
+    ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_CRY,
+                         NGTCP2_LOG_TP
+                         " original_destination_connection_id=0x%s",
+                         ngtcp2_encode_hex_cstr(cid, params->original_dcid.data,
+                                                params->original_dcid.datalen));
   }
 
   if (params->retry_scid_present) {
-    ngtcp2_log_infof_raw(
-      log, NGTCP2_LOG_EVENT_CRY,
-      NGTCP2_LOG_TP " retry_source_connection_id=0x%s",
-      (const char *)ngtcp2_encode_hex(cid, params->retry_scid.data,
-                                      params->retry_scid.datalen));
+    ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_CRY,
+                         NGTCP2_LOG_TP " retry_source_connection_id=0x%s",
+                         ngtcp2_encode_hex_cstr(cid, params->retry_scid.data,
+                                                params->retry_scid.datalen));
   }
 
   if (params->initial_scid_present) {
-    ngtcp2_log_infof_raw(
-      log, NGTCP2_LOG_EVENT_CRY,
-      NGTCP2_LOG_TP " initial_source_connection_id=0x%s",
-      (const char *)ngtcp2_encode_hex(cid, params->initial_scid.data,
-                                      params->initial_scid.datalen));
+    ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_CRY,
+                         NGTCP2_LOG_TP " initial_source_connection_id=0x%s",
+                         ngtcp2_encode_hex_cstr(cid, params->initial_scid.data,
+                                                params->initial_scid.datalen));
   }
 
   ngtcp2_log_infof_raw(log, NGTCP2_LOG_EVENT_CRY,
@@ -732,8 +734,8 @@ void ngtcp2_log_pkt_lost(ngtcp2_log *log, int64_t pkt_num, uint8_t type,
 
 static void log_pkt_hd(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
                        const char *dir) {
-  uint8_t dcid[sizeof(hd->dcid.data) * 2 + 1];
-  uint8_t scid[sizeof(hd->scid.data) * 2 + 1];
+  char dcid[sizeof(hd->dcid.data) * 2 + 1];
+  char scid[sizeof(hd->scid.data) * 2 + 1];
 
   if (!log->log_printf || !(log->events & NGTCP2_LOG_EVENT_PKT)) {
     return;
@@ -743,15 +745,15 @@ static void log_pkt_hd(ngtcp2_log *log, const ngtcp2_pkt_hd *hd,
     ngtcp2_log_infof(
       log, NGTCP2_LOG_EVENT_PKT, "%s pkn=%" PRId64 " dcid=0x%s type=%s k=%d",
       dir, hd->pkt_num,
-      (const char *)ngtcp2_encode_hex(dcid, hd->dcid.data, hd->dcid.datalen),
+      ngtcp2_encode_hex_cstr(dcid, hd->dcid.data, hd->dcid.datalen),
       strpkttype(hd), (hd->flags & NGTCP2_PKT_FLAG_KEY_PHASE) != 0);
   } else {
     ngtcp2_log_infof(
       log, NGTCP2_LOG_EVENT_PKT,
       "%s pkn=%" PRId64 " dcid=0x%s scid=0x%s version=0x%08x type=%s len=%zu",
       dir, hd->pkt_num,
-      (const char *)ngtcp2_encode_hex(dcid, hd->dcid.data, hd->dcid.datalen),
-      (const char *)ngtcp2_encode_hex(scid, hd->scid.data, hd->scid.datalen),
+      ngtcp2_encode_hex_cstr(dcid, hd->dcid.data, hd->dcid.datalen),
+      ngtcp2_encode_hex_cstr(scid, hd->scid.data, hd->scid.datalen),
       hd->version, strpkttype(hd), hd->len);
   }
 }
